@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2006 Zope Corporation and Contributors.
+# Copyright (c) 2006-2007 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -11,33 +11,29 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Grok components
-"""
+"""Grok components"""
 
+import sys
 import os
 import persistent
-import urllib
 import datetime
 import warnings
 import pytz
 import simplejson
 
+import zope.location
 from zope import component
 from zope import interface
-from zope import schema
-from zope import event
 from zope.interface.common import idatetime
-from zope.lifecycleevent import ObjectModifiedEvent
+from zope.security.permission import Permission
+from zope.securitypolicy.role import Role
 from zope.publisher.browser import BrowserPage
 from zope.publisher.interfaces import NotFound
-from zope.publisher.interfaces.browser import (IBrowserPublisher,
-                                               IBrowserRequest)
+from zope.publisher.interfaces.browser import IBrowserPublisher
+from zope.publisher.interfaces.http import IHTTPRequest
 from zope.publisher.publish import mapply
 from zope.pagetemplate import pagetemplate, pagetemplatefile
 from zope.formlib import form
-from zope.traversing.browser.interfaces import IAbsoluteURL
-from zope.traversing.browser.absoluteurl import AbsoluteURL
-from zope.traversing.browser.absoluteurl import _safe as SAFE_URL_CHARACTERS
 from zope.annotation.interfaces import IAttributeAnnotatable
 
 from zope.app.pagetemplate.engine import TrustedAppPT
@@ -47,107 +43,113 @@ from zope.app.publisher.browser.pagetemplateresource import \
     PageTemplateResourceFactory
 from zope.app.container.btree import BTreeContainer
 from zope.app.container.contained import Contained
-from zope.app.container.interfaces import IReadContainer
+from zope.app.container.interfaces import IReadContainer, IObjectAddedEvent
+from zope.app.container.interfaces import IOrderedContainer
+from zope.app.container.contained import notifyContainerModified
+from persistent.list import PersistentList
 from zope.app.component.site import SiteManagerContainer
+from zope.app.component.site import LocalSiteManager
 
-from grok import util, interfaces, formlib
+from zope.viewlet.manager import ViewletManagerBase
+from zope.viewlet.viewlet import ViewletBase
 
+import grok
+import grokcore.component
+import z3c.flashmessage.interfaces
+import martian.util
 
-# These base grokkers exist in grok.components because they are meant
-# to be subclassed by code that extends grok.  Thus they are like
-# grok.Model, grok.View, etc. in that they should not be grokked
-# themselves but subclasses of them.
-
-class GrokkerBase(object):
-    """A common base class for all grokkers.
-    """
-
-    priority = 0
-    continue_scanning = False
-
-
-class ClassGrokker(GrokkerBase):
-    """Grokker for particular classes in a module.
-    """
-    # subclasses should have a component_class class variable
-
-    def match(self, obj):
-        return util.check_subclass(obj, self.component_class)
-
-    def register(self, context, name, factory, module_info, templates):
-        raise NotImplementedError
-
-
-class InstanceGrokker(GrokkerBase):
-    """Grokker for particular instances in a module.
-    """
-    # subclasses should have a component_class class variable
-
-    def match(self, obj):
-        return isinstance(obj, self.component_class)
-
-    def register(self, context, name, instance, module_info, templates):
-        raise NotImplementedError
-
-
-class ModuleGrokker(GrokkerBase):
-    """Grokker that gets executed once for a module.
-    """
-
-    def match(self, obj):
-        # we never match with any object
-        return False
-
-    def register(self, context, module_info, templates):
-        raise NotImplementedError
+from grok import interfaces, formlib, util
 
 
 class Model(Contained, persistent.Persistent):
     # XXX Inheritance order is important here. If we reverse this,
     # then containers can't be models anymore because no unambigous MRO
     # can be established.
-    interface.implements(IAttributeAnnotatable)
+    interface.implements(IAttributeAnnotatable, interfaces.IContext)
 
 
 class Container(BTreeContainer):
-    interface.implements(IAttributeAnnotatable)
+    interface.implements(IAttributeAnnotatable, interfaces.IContainer)
+
+
+class OrderedContainer(Container):
+    interface.implements(IOrderedContainer)
+
+    def __init__(self):
+        super(OrderedContainer, self).__init__()
+        self._order = PersistentList()
+
+    def keys(self):
+        # Return a copy of the list to prevent accidental modifications.
+        return self._order[:]
+
+    def __iter__(self):
+        return iter(self.keys())
+
+    def values(self):
+        return (self[key] for key in self._order)
+
+    def items(self):
+        return ((key, self[key]) for key in self._order)
+
+    def __setitem__(self, key, object):
+        foo = self.has_key(key)
+        # Then do whatever containers normally do.
+        super(OrderedContainer, self).__setitem__(key, object)
+        if not foo:
+            self._order.append(key)
+
+    def __delitem__(self, key):
+        # First do whatever containers normally do.
+        super(OrderedContainer, self).__delitem__(key)
+        self._order.remove(key)
+
+    def updateOrder(self, order):
+        if set(order) != set(self._order):
+            raise ValueError("Incompatible key set.")
+
+        self._order = PersistentList()
+        self._order.extend(order)
+        notifyContainerModified(self)
 
 
 class Site(SiteManagerContainer):
     pass
 
+@component.adapter(Site, IObjectAddedEvent)
+def addSiteHandler(site, event):
+    sitemanager = LocalSiteManager(site)
+    # LocalSiteManager creates the 'default' folder in its __init__.
+    # It's not needed anymore in new versions of Zope 3, therefore we
+    # remove it
+    del sitemanager['default']
+    site.setSiteManager(sitemanager)
+
 
 class Application(Site):
     """A top-level application object."""
     interface.implements(interfaces.IApplication)
 
-class Adapter(object):
-
-    def __init__(self, context):
-        self.context = context
-
-
-class GlobalUtility(object):
-    pass
-
 
 class LocalUtility(Model):
     pass
 
 
-class MultiAdapter(object):
-    pass
-
-
 class Annotation(persistent.Persistent):
     pass
 
 
+class ViewBase(object):
+    def __init__(self, context, request):
+        self.context = context
+        self.request = request
+
 class View(BrowserPage):
     interface.implements(interfaces.IGrokView)
 
     def __init__(self, context, request):
         super(View, self).__init__(context, request)
+        self.__name__ = self.__view_name__
         self.static = component.queryAdapter(
             self.request,
             interface.Interface,
@@ -171,21 +173,38 @@ class View(BrowserPage):
         return mapply(self.render, (), self.request)
 
     def _render_template(self):
-        namespace = self.template.pt_getContext()
-        namespace['request'] = self.request
-        namespace['view'] = self
+        return self.template.render(self)
+
+    def default_namespace(self):
+        namespace = {}
         namespace['context'] = self.context
-        # XXX need to check whether we really want to put None here if missing
+        namespace['request'] = self.request
         namespace['static'] = self.static
-        return self.template.pt_render(namespace)
+        namespace['view'] = self
+        return namespace
+
+    def namespace(self):
+        return {}
 
     def __getitem__(self, key):
-        # XXX give nice error message if template is None
-        return self.template.macros[key]
+        # This is BBB code for Zope page templates only:
+        if not isinstance(self.template, PageTemplate):
+            raise AttributeError("View has no item %s" % key)
 
-    def url(self, obj=None, name=None):
-        # if the first argument is a string, that's the name. There should
-        # be no second argument
+        value = self.template._template.macros[key]
+        # When this deprecation is done with, this whole __getitem__ can
+        # be removed.
+        warnings.warn("Calling macros directly on the view is deprecated. "
+                      "Please use context/@@viewname/macros/macroname\n"
+                      "View %r, macro %s" % (self, key),
+                      DeprecationWarning, 1)
+        return value
+
+
+    def url(self, obj=None, name=None, data=None):
+        """Return string for the URL based on the obj and name. The data
+        argument is used to form a CGI query string.
+        """
         if isinstance(obj, basestring):
             if name is not None:
                 raise TypeError(
@@ -200,7 +219,22 @@ class View(BrowserPage):
         elif name is not None and obj is None:
             # create URL to view on context
             obj = self.context
-        return util.url(self.request, obj, name)
+
+        if data is None:
+            data = {}
+        else:
+            if not isinstance(data, dict):
+                raise TypeError('url() data argument must be a dict.')
+
+        return util.url(self.request, obj, name, data=data)
+
+    def application_url(self, name=None):
+        obj = self.context
+        while obj is not None:
+            if isinstance(obj, Application):
+                return self.url(obj, name)
+            obj = obj.__parent__
+        raise ValueError("No application found.")
 
     def redirect(self, url):
         return self.request.response.redirect(url)
@@ -208,17 +242,39 @@ class View(BrowserPage):
     def update(self):
         pass
 
-class GrokViewAbsoluteURL(AbsoluteURL):
-
-    def _getContextName(self, context):
-        return getattr(context, '__view_name__', None)
-    # XXX breadcrumbs method on AbsoluteURL breaks as it does not use
-    # _getContextName to get to the name of the view. What does breadcrumbs do?
+    def flash(self, message, type='message'):
+        source = component.getUtility(
+            z3c.flashmessage.interfaces.IMessageSource, name='session')
+        source.send(message, type)
 
 
 class XMLRPC(object):
     pass
 
+class REST(zope.location.Location):
+    interface.implements(interfaces.IREST)
+
+    def __init__(self, context, request):
+        self.context = self.__parent__ = context
+        self.request = request
+        self.body = request.bodyStream.getCacheStream().read()
+
+    @property
+    def response(self):
+        return self.request.response
+
+##     def GET(self):
+##         raise GrokMethodNotAllowed(self.context, self.request)
+
+##     def POST(self):
+##         raise GrokMethodNotAllowed(self.context, self.request)
+
+##     def PUT(self):
+##         raise GrokMethodNotAllowed(self.context, self.request)
+
+##     def DELETE(self):
+##         raise GrokMethodNotAllowed(self.context, self.request)
+
 class JSON(BrowserPage):
 
     def __call__(self):
@@ -227,7 +283,14 @@ class JSON(BrowserPage):
         method_result = mapply(method, (), self.request)
         return simplejson.dumps(method_result)
 
-class GrokPageTemplate(object):
+
+class BaseTemplate(object):
+    """Any sort of page template"""
+
+    interface.implements(interfaces.ITemplate)
+
+    __grok_name__ = ''
+    __grok_location__ = ''
 
     def __repr__(self):
         return '<%s template in %s>' % (self.__grok_name__,
@@ -237,37 +300,94 @@ class GrokPageTemplate(object):
         self.__grok_name__ = name
         self.__grok_location__ = location
 
+    def _initFactory(self, factory):
+        pass
 
-class PageTemplate(GrokPageTemplate, TrustedAppPT, pagetemplate.PageTemplate):
-    expand = 0
 
-    def __init__(self, template):
-        super(PageTemplate, self).__init__()
-        if util.not_unicode_or_ascii(template):
+class GrokTemplate(BaseTemplate):
+    """A slightly more advanced page template
+
+    This provides most of what a page template needs and is a good base for
+    writing your own page template"""
+
+    def __init__(self, string=None, filename=None, _prefix=None):
+
+        # __grok_module__ is needed to make defined_locally() return True for
+        # inline templates
+        # XXX unfortunately using caller_module means that care must be taken
+        # when GrokTemplate is subclassed. You can not do a super().__init__
+        # for example.
+        self.__grok_module__ = martian.util.caller_module()
+
+        if not (string is None) ^ (filename is None):
+            raise AssertionError(
+                "You must pass in template or filename, but not both.")
+
+        if string:
+            self.setFromString(string)
+        else:
+            if _prefix is None:
+                module = sys.modules[self.__grok_module__]
+                _prefix = os.path.dirname(module.__file__)
+            self.setFromFilename(filename, _prefix)
+
+    def __repr__(self):
+        return '<%s template in %s>' % (self.__grok_name__,
+                                        self.__grok_location__)
+
+    def _annotateGrokInfo(self, name, location):
+        self.__grok_name__ = name
+        self.__grok_location__ = location
+
+    def _initFactory(self, factory):
+        pass
+
+    def namespace(self, view):
+        # By default use the namespaces that are defined as the
+        # default by the view implementation.
+        return view.default_namespace()
+
+    def getNamespace(self, view):
+        namespace = self.namespace(view)
+        namespace.update(view.namespace())
+        return namespace
+
+class TrustedPageTemplate(TrustedAppPT, pagetemplate.PageTemplate):
+    pass
+
+class TrustedFilePageTemplate(TrustedAppPT, pagetemplatefile.PageTemplateFile):
+    pass
+
+class PageTemplate(GrokTemplate):
+
+    def setFromString(self, string):
+        zpt = TrustedPageTemplate()
+        if martian.util.not_unicode_or_ascii(string):
             raise ValueError("Invalid page template. Page templates must be "
                              "unicode or ASCII.")
-        self.write(template)
+        zpt.write(string)
+        self._template = zpt
 
-        # __grok_module__ is needed to make defined_locally() return True for
-        # inline templates
-        # XXX unfortunately using caller_module means that
-        # PageTemplate cannot be subclassed
-        self.__grok_module__ = util.caller_module()
+    def setFromFilename(self, filename, _prefix=None):
+        self._template = TrustedFilePageTemplate(filename, _prefix)
 
+    def _initFactory(self, factory):
+        factory.macros = self._template.macros
 
-class PageTemplateFile(GrokPageTemplate, TrustedAppPT,
-                       pagetemplatefile.PageTemplateFile):
+    def render(self, view):
+        namespace = self.getNamespace(view)
+        template = self._template
+        namespace.update(template.pt_getContext())
+        return template.pt_render(namespace)
 
+class PageTemplateFile(PageTemplate):
+    # For BBB
     def __init__(self, filename, _prefix=None):
-        _prefix = self.get_path_from_prefix(_prefix)
-        super(PageTemplateFile, self).__init__(filename, _prefix)
-
-        # __grok_module__ is needed to make defined_locally() return True for
-        # inline templates
-        # XXX unfortunately using caller_module means that
-        # PageTemplateFile cannot be subclassed
-        self.__grok_module__ = util.caller_module()
-
+        self.__grok_module__ = martian.util.caller_module()
+        if _prefix is None:
+            module = sys.modules[self.__grok_module__]
+            _prefix = os.path.dirname(module.__file__)
+        self.setFromFilename(filename, _prefix)
 
 class DirectoryResource(directoryresource.DirectoryResource):
     # We subclass this, because we want to override the default factories for
@@ -305,6 +425,14 @@ class Traverser(object):
         self.request = request
 
     def browserDefault(self, request):
+        # if we have a RESTful request, we will handle
+        # GET, POST and HEAD differently (PUT and DELETE are handled already
+        # but not on the BrowserRequest layer but the HTTPRequest layer)
+        if IRESTLayer.providedBy(request):
+            rest_view = component.getMultiAdapter(
+                (self.context, self.request),
+                name=request.method)
+            return rest_view, ()
         view_name = getDefaultViewName(self.context, request)
         view_uri = "@@%s" % view_name
         return self.context, (view_uri,)
@@ -312,7 +440,15 @@ class Traverser(object):
     def publishTraverse(self, request, name):
         subob = self.traverse(name)
         if subob is not None:
-            return subob
+            return util.safely_locate_maybe(subob, self.context, name)
+
+        traversable_dict = grok.traversable.bind().get(self.context)
+        if traversable_dict:
+            if name in traversable_dict:
+                subob = getattr(self.context, traversable_dict[name])
+                if callable(subob):
+                    subob = subob()
+                return util.safely_locate_maybe(subob, self.context, name)
 
         # XXX Special logic here to deal with containers.  It would be
         # good if we wouldn't have to do this here. One solution is to
@@ -334,8 +470,8 @@ class Traverser(object):
         pass
 
 
-class ModelTraverser(Traverser):
-    component.adapts(Model, IBrowserRequest)
+class ContextTraverser(Traverser):
+    component.adapts(interfaces.IContext, IHTTPRequest)
 
     def traverse(self, name):
         traverse = getattr(self.context, 'traverse', None)
@@ -344,7 +480,7 @@ class ModelTraverser(Traverser):
 
 
 class ContainerTraverser(Traverser):
-    component.adapts(Container, IBrowserRequest)
+    component.adapts(interfaces.IContainer, IHTTPRequest)
 
     def traverse(self, name):
         traverse = getattr(self.context, 'traverse', None)
@@ -355,6 +491,7 @@ class ContainerTraverser(Traverser):
         # try to get the item from the container
         return self.context.get(name)
 
+
 default_form_template = PageTemplateFile(os.path.join(
     'templates', 'default_edit_form.pt'))
 default_form_template.__grok_name__ = 'default_edit_form'
@@ -362,11 +499,12 @@ default_display_template = PageTemplateFile(os.path.join(
     'templates', 'default_display_form.pt'))
 default_display_template.__grok_name__ = 'default_display_form'
 
+
 class GrokForm(object):
-    """Mix-in to console zope.formlib's forms with grok.View and to
+    """Mix-in to consolidate zope.formlib's forms with grok.View and to
     add some more useful methods.
 
-    The consolation needs to happen because zope.formlib's Forms have
+    The consolidation needs to happen because zope.formlib's Forms have
     update/render methods which have different meanings than
     grok.View's update/render methods.  We deal with this issue by
     'renaming' zope.formlib's update() to update_form() and by
@@ -414,6 +552,7 @@ class GrokForm(object):
         self.update_form()
         return self.render()
 
+
 class Form(GrokForm, form.FormBase, View):
     # We're only reusing the form implementation from zope.formlib, we
     # explicitly don't want to inherit the interface semantics (mostly
@@ -433,9 +572,11 @@ class Form(GrokForm, form.FormBase, View):
                       "'applyData' instead.", DeprecationWarning, 2)
         return bool(self.applyData(obj, **data))
 
+
 class AddForm(Form):
     pass
 
+
 class EditForm(GrokForm, form.EditFormBase, View):
     # We're only reusing the form implementation from zope.formlib, we
     # explicitly don't want to inherit the interface semantics (mostly
@@ -472,6 +613,7 @@ class EditForm(GrokForm, form.EditFormBase, View):
         else:
             self.status = 'No changes'
 
+
 class DisplayForm(GrokForm, form.DisplayFormBase, View):
     # We're only reusing the form implementation from zope.formlib, we
     # explicitly don't want to inherit the interface semantics (mostly
@@ -480,25 +622,142 @@ class DisplayForm(GrokForm, form.DisplayFormBase, View):
 
     template = default_display_template
 
+
 class IndexesClass(object):
     def __init__(self, name, bases=(), attrs=None):
         if attrs is None:
             return
-        # make sure we take over a bunch of possible attributes
-        for name in ['__grok_context__', '__grok_name__',
-                     '__grok_site__']:
-            value = attrs.get(name)
-            if value is not None:
-                setattr(self, name, value)
-        # now read and store indexes
         indexes = {}
         for name, value in attrs.items():
+            # Ignore everything that's not an index definition object
+            # except for values set by directives
+            if '.' in name:
+                setattr(self, name, value)
+                continue
             if not interfaces.IIndexDefinition.providedBy(value):
                 continue
             indexes[name] = value
         self.__grok_indexes__ = indexes
         # __grok_module__ is needed to make defined_locally() return True for
         # inline templates
-        self.__grok_module__ = util.caller_module()
-        
+        self.__grok_module__ = martian.util.caller_module()
+
 Indexes = IndexesClass('Indexes')
+
+class Permission(Permission):
+    pass
+
+Public = 'zope.Public'
+
+class Role(Role):
+    pass
+
+class IGrokLayer(interface.Interface):
+    pass
+
+class IRESTLayer(interface.Interface):
+    pass
+
+class Skin(object):
+    pass
+
+class RESTProtocol(object):
+    pass
+
+class ViewletManager(ViewletManagerBase):
+    interface.implements(interfaces.IViewletManager)
+
+    template = None
+
+    def __init__(self, context, request, view):
+        super(ViewletManager, self).__init__(context, request, view)
+        self.context = context
+        self.request = request
+        self.view = view
+        self.__name__ = self.__view_name__
+        self.static = component.queryAdapter(
+            self.request,
+            interface.Interface,
+            name=self.module_info.package_dotted_name
+            )
+
+    def sort(self, viewlets):
+        """Sort the viewlets.
+
+        ``viewlets`` is a list of tuples of the form (name, viewlet).
+        """
+        # In Grok, the default order of the viewlets is determined by
+        # util.sort_components. util.sort_components() however expects
+        # a list of just components, but sort() is supposed to deal
+        # with a list of (name, viewlet) tuples.
+        # To handle this situation we first store the name part on the
+        # viewlet, then use util.sort_components() and then "unpack"
+        # the name from the viewlet and recreate the list of (name,
+        # viewlet) tuples, now in the correct order.
+        s_viewlets = []
+        for name, viewlet in viewlets:
+             # Stuff away viewlet name so we can later retrieve it.
+             # XXX We loose name information in case the same viewlet
+             # is in the viewlets list twice, but with a different
+             # name. Most probably this situation doesn't occur.
+             viewlet.__viewlet_name__ = name
+             s_viewlets.append(viewlet)
+        s_viewlets = util.sort_components(s_viewlets)
+        return [(viewlet.__viewlet_name__, viewlet) for viewlet in s_viewlets]
+
+    def default_namespace(self):
+        namespace = {}
+        namespace['context'] = self.context
+        namespace['request'] = self.request
+        namespace['static'] = self.static
+        namespace['view'] = self.view
+        namespace['viewletmanager'] = self
+        return namespace
+
+    def namespace(self):
+        return {}
+
+    def render(self):
+        """See zope.contentprovider.interfaces.IContentProvider"""
+        # Now render the view
+        if self.template:
+            return self.template.render(self)
+        else:
+            return u'\n'.join([viewlet.render() for viewlet in self.viewlets])
+
+
+class Viewlet(ViewletBase):
+    """Batteries included viewlet.
+    """
+
+    def __init__(self, context, request, view, manager):
+        super(Viewlet, self).__init__(context, request, view, manager)
+        self.context = context
+        self.request = request
+        self.view = view
+        self.viewletmanager = manager
+        self.__name__ = self.__view_name__
+        self.static = component.queryAdapter(
+            self.request,
+            interface.Interface,
+            name=self.module_info.package_dotted_name
+            )
+
+    def default_namespace(self):
+        namespace = {}
+        namespace['context'] = self.context
+        namespace['request'] = self.request
+        namespace['static'] = self.static
+        namespace['view'] = self.view
+        namespace['viewlet'] = self
+        namespace['viewletmanager'] = self.manager
+        return namespace
+
+    def namespace(self):
+        return {}
+
+    def update(self):
+        pass
+
+    def render(self):
+        return self.template.render(self)
