@@ -1,16 +1,16 @@
 """
 A grok.Fields can receive keyword parameters with schema fields. These
-should be avaible in the definition order.
+should be available in the definition order.
 
-  >>> grok.grok(__name__)
+  >>> grok.testing.grok(__name__)
 
   >>> from zope import component
   >>> from zope.publisher.browser import TestRequest
   >>> request = TestRequest()
   >>> view = component.getMultiAdapter((Mammoth(), request), name='edit')
-  >>> len(view.form.form_fields)
+  >>> len(view.form_fields)
   4
-  >>> [w.__name__ for w in view.form.form_fields]
+  >>> [w.__name__ for w in view.form_fields]
   ['a', 'b', 'g', 'd']
 
 """
