@@ -1,7 +1,7 @@
 """
 Using the @grok.require decorator in a view class is not allowed.
 
-  >>> grok.grok(__name__)
+  >>> grok.testing.grok(__name__)
   Traceback (most recent call last):
   GrokError: The @grok.require decorator is used for method 'render' in view <class 'grok.tests.security.view_decorator.BogusView'>. It may only be used for XML-RPC methods.
 
@@ -11,11 +11,12 @@ Using the @grok.require decorator in a view class is not allowed.
 import grok
 import zope.interface
 
-grok.define_permission('bogus.perm')
+class Bogus(grok.Permission):
+    grok.name('bogus.perm')
 
 class BogusView(grok.View):
     grok.context(zope.interface.Interface)
 
-    @grok.require('bogus.perm')
+    @grok.require(Bogus)
     def render(self):
         pass
