@@ -1,6 +1,7 @@
+import re
 import unittest
 from pkg_resources import resource_listdir
-from zope.testing import doctest, cleanup
+from zope.testing import doctest, cleanup, renormalizing
 import zope.component.eventtesting
 
 def setUpZope(test):
@@ -9,6 +10,14 @@ def setUpZope(test):
 def cleanUpZope(test):
     cleanup.cleanUp()
 
+checker = renormalizing.RENormalizing([
+    # str(Exception) has changed from Python 2.4 to 2.5 (due to
+    # Exception now being a new-style class).  This changes the way
+    # exceptions appear in traceback printouts.
+    (re.compile(r"ConfigurationExecutionError: <class '([\w.]+)'>:"),
+                r'ConfigurationExecutionError: \1:'),
+    ])
+
 def suiteFromPackage(name):
     files = resource_listdir(__name__, name)
     suite = unittest.TestSuite()
@@ -24,6 +33,7 @@ def suiteFromPackage(name):
         test = doctest.DocTestSuite(dottedname,
                                     setUp=setUpZope,
                                     tearDown=cleanUpZope,
+                                    checker=checker,
                                     optionflags=doctest.ELLIPSIS+
                                     doctest.NORMALIZE_WHITESPACE)
 
@@ -32,10 +42,11 @@ def suiteFromPackage(name):
 
 def test_suite():
     suite = unittest.TestSuite()
-    for name in ['adapter', 'error', 'view', 'scan', 'event', 'security',
-                 'zcml', 'static', 'utility', 'xmlrpc', 'container',
-                 'traversal', 'form', 'site', 'grokker', 'directive', 'util',
-                 'baseclass', 'annotation', 'application']:
+    for name in ['adapter', 'error', 'view', 'event', 'security', 'catalog',
+                 'zcml', 'static', 'utility', 'xmlrpc', 'json', 'container',
+                 'traversal', 'form', 'grokker', 'directive',
+                 'baseclass', 'annotation', 'application', 'template',
+                 'viewlet', 'testsetup', 'conflict', 'order']:
         suite.addTest(suiteFromPackage(name))
     return suite
 
