@@ -1,7 +1,7 @@
 """
 Subclasses of grok.GlobalUtility must implement exactly one interface:
 
-  >>> grok.grok(__name__)
+  >>> grok.testing.grok(__name__)
   Traceback (most recent call last):
     ...
   GrokError: <class 'grok.tests.utility.implementsnone2.Club'> must
