@@ -2,12 +2,12 @@
 This should fail because ``grok.template`` points to a non-existing
 template:
 
-  >>> grok.grok(__name__)
+  >>> grok.testing.grok(__name__)
   Traceback (most recent call last):
     ...
-  GrokError: View <class 'grok.tests.view.templatenotfound.Painting'>
+  ConfigurationExecutionError: martian.error.GrokError: View <class 'grok.tests.view.templatenotfound.Painting'>
   has no associated template or 'render' method.
-
+  in:
 """
 import grok
 
